# -*- coding: utf-8 -*-


class ASF_Error(Exception):

    def __init__(self, message):
        super(ASF_Error, self).__init__()
        self.message = message

    def __str__(self):
        return self.message


class ASF_BadRequest(ASF_Error):

    def __init__(self, message):
        super(ASF_BadRequest, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_Unauthorized(ASF_Error):

    def __init__(self, message):
        super(ASF_Unauthorized, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_Forbidden(ASF_Error):

    def __init__(self, message):
        super(ASF_Forbidden, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_NotFound(ASF_Error):

    def __init__(self, message):
        super(ASF_NotFound, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_NotAllowed(ASF_Error):

    def __init__(self, message):
        super(ASF_NotAllowed, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_NotAcceptable(ASF_Error):

    def __init__(self, message):
        super(ASF_NotAcceptable, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_LengthRequired(ASF_Error):

    def __init__(self, message):
        super(ASF_LengthRequired, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_InternalServerError(ASF_Error):

    def __init__(self, message):
        super(ASF_InternalServerError, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message


class ASF_NotImplemented(ASF_Error):

    def __init__(self, message):
        super(ASF_NotImplemented, self).__init__(message)
        self.message = message

    def __str__(self):
        return self.message



# -*- coding: utf-8 -*-
import urllib.parse as urlparse
import requests

from . import response

ALLBOT = 'ASF'


class IPC(object):

    def __init__(self, host='127.0.0.1', port=1242, password='', timeout=5):
        self.host = host
        self.port = port
        self.password = password
        self.root_url = 'http://{0}:{1}'.format(host, port)
        self.timeout = timeout

    @classmethod
    def _botjoin(cls, bots):
        botnames = bots
        if isinstance(botnames, (list, tuple)):
            botnames = ','.join(botnames)
        botnames = urlparse.quote_plus(botnames)
        return botnames

    def _add_auth(self, headers=None):
        if headers is None:
            auth_headers = dict()
        else:
            auth_headers = headers
        if self.password:
            auth_headers['Authentication'] = self.password
        return auth_headers

    def _get(self, url, headers=None):
        headers = self._add_auth(headers)
        resp = requests.get(url, headers=headers, timeout=self.timeout)
        resp = response.GenericResponse(resp)
        if not resp.success:
            raise Exception(resp.error)
        return resp.result

    def _delete(self, url, headers=None):
        headers = self._add_auth(headers)
        resp = requests.delete(url, headers=headers, timeout=self.timeout)
        resp = response.GenericResponse(resp)
        if not resp.success:
            raise Exception(resp.error)

    def _post(self, url, data=None, headers=None):
        headers = self._add_auth(headers)
        resp = requests.post(url, json=data, headers=headers, timeout=self.timeout)
        resp = response.GenericResponse(resp)
        if not resp.success:
            raise Exception(resp.error)
        return resp.result

    def get_bot(self, botnames, key=None):
        botnames = self._botjoin(botnames)
        address = self.root_url + '/Api/Bot/' + botnames
        return self._get(address)

    def delete_bot(self, botnames):
        botnames = self._botjoin(botnames)
        address = self.root_url + '/Api/Bot/' + botnames
        self._delete(address)

    def post_bot(self, botname, config, keep_sensitive=True):
        botname = urlparse.quote_plus(botname)
        address = self.root_url + '/Api/Bot/' + botname
        headers = {'Content-Type': 'application/json'}
        payload = {
            'BotConfig': config,
            'KeepSensitiveDetails': keep_sensitive
        }
        self._post(address, payload, headers)

    def command(self, cmd):
        cmd = urlparse.quote_plus(cmd)
        address = self.root_url + '/Api/Command/' + cmd
        return self._post(address)

    def get_structure(self, name):
        name = urlparse.quote_plus(name)
        address = self.root_url + '/Api/Structure/' + name
        return self._get(address)

    def get_type(self, name):
        name = urlparse.quote_plus(name)
        address = self.root_url + '/Api/Type/' + name
        return self._get(address)



# -*- coding: utf-8 -*-
from http.client import responses

from . import error


class GenericResponse(object):

    exceptions = {
        400: error.ASF_BadRequest,
        401: error.ASF_Unauthorized,
        403: error.ASF_Forbidden,
        404: error.ASF_NotFound,
        405: error.ASF_NotAllowed,
        406: error.ASF_NotAcceptable,
        411: error.ASF_LengthRequired,
        500: error.ASF_InternalServerError,
        501: error.ASF_NotImplemented
    }

    def __init__(self, response):
        self.code = response.status_code
        try:
            json_content = response.json()
            self.message = json_content.get('Message')
            self.result = json_content.get('Result')
            self.success = json_content.get('Success')
            if not self.success:
                self.error = self.message
        except ValueError:
            self.message = ''
            self.result = response.text
            self.success = False
            self.error = '{0} - {1}'.format(self.code, responses[self.code])
        if not self.success:
            if self.code in self.exceptions:
                raise self.exceptions[self.code](self.error)
            else:
                raise error.ASF_Error(self.error)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='ASF_IPC',
    version='1.0.4',
    packages=['ASF_IPC'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'requests'
    ],
    author='deluxghost',
    author_email='deluxghost@gmail.com',
    description='A simple Python 3 library of ArchiSteamFarm IPC API',
    long_description='Check GitHub page for usages.',
    license='GPL v3',
    url='https://github.com/deluxghost/ASF_IPC/'
)
